use std::marker::ConstParamTy;

#[derive(ConstParamTy, PartialEq, Eq, Clone, Copy)]
pub enum Level {
    MAYO1,
    MAYO2,
    MAYO3,
    MAYO5,
}

pub const fn n(level: Level) -> usize {
    match level {
        Level::MAYO1 => 66,
        Level::MAYO2 => 78,
        Level::MAYO3 => 99,
        Level::MAYO5 => 133,
    }
}

pub const fn m(level: Level) -> usize {
    match level {
        Level::MAYO1 => 64,
        Level::MAYO2 => 64,
        Level::MAYO3 => 99,
        Level::MAYO5 => 128,
    }
}

pub const fn o(level: Level) -> usize {
    match level {
        Level::MAYO1 => 8,
        Level::MAYO2 => 18,
        Level::MAYO3 => 10,
        Level::MAYO5 => 12,
    }
}

pub const fn k(level: Level) -> usize {
    match level {
        Level::MAYO1 => 9,
        Level::MAYO2 => 4,
        Level::MAYO3 => 11,
        Level::MAYO5 => 12,
    }
}

pub const fn q(level: Level) -> usize {
    16
}

pub const fn salt_bytes(level: Level) -> usize {
    match level {
        Level::MAYO1 => 24,
        Level::MAYO2 => 24,
        Level::MAYO3 => 32,
        Level::MAYO5 => 40,
    }
}

pub const fn digest_bytes(level: Level) -> usize {
    match level {
        Level::MAYO1 => 32,
        Level::MAYO2 => 32,
        Level::MAYO3 => 48,
        Level::MAYO5 => 64,
    }
}

pub const fn pk_seed_bytes(_level: Level) -> usize {
    16
}

pub const fn poly(level: Level) -> usize {
    match level {
        Level::MAYO1 => 64,
        Level::MAYO2 => 64,
        Level::MAYO3 => 96,
        Level::MAYO5 => 128,
    }
}

pub const fn polymat<const L: Level>() -> [[u8; m(L)]; m(L)] {
    match poly(L) {
        64 => {
            todo!()
        }
        96 => {
            todo!()
        }
        128 => {
            todo!()
        }
        _ => unimplemented!(),
    }
}

pub const fn v_bytes(level: Level) -> usize {
    (n(level) - o(level) + 1) / 2
}

pub const fn o_bytes(level: Level) -> usize {
    (((n(level) - o(level)) * o(level)) + 1) / 2
}

pub const fn p1_bytes(level: Level) -> usize {
    m(level) * ((n(level) - o(level) + 1) * (n(level) - o(level))) / 2
}

pub const fn p2_bytes(level: Level) -> usize {
    m(level) * (n(level) - o(level)) * o(level) / 2
}

pub const fn p3_bytes(level: Level) -> usize {
    m(level) * (o(level) + 1) * (o(level)) / 4
}

pub const fn l_bytes(level: Level) -> usize {
    m(level) * (n(level) - o(level)) * o(level) / 2
}
