use crypto::aes::{self, KeySize};
use tiny_keccak::{Hasher, Shake};

pub fn aes_128_ctr(seed: &[u8], l: usize) -> Vec<u8> {
    let nonce = [0u8; 16];
    let mut cipher = aes::ctr(KeySize::KeySize128, seed, &nonce);
    let enc = vec![0u8; l];
    let mut out = vec![0u8; l];
    cipher.process(&enc, &mut out[..]);
    out
}

pub fn shake_256<const S: usize>(inputs: Vec<&[u8]>) -> [u8; S] {
    let mut shake = Shake::v256();

    for inp in inputs {
        shake.update(inp);
    }

    let mut out: [u8; S] = [0; S];

    shake.finalize(&mut out);

    out
}
